@@ -16,7 +16,7 @@
 
 from __future__ import print_function, unicode_literals, division, absolute_import
 
-import os, sys, io, locale, threading
+import os, sys, io, locale, threading, hashlib
 from io import TextIOWrapper
 from contextlib import contextmanager
 try:
@@ -220,3 +220,11 @@ def unwrap_stream(stream_name):
     finally:
         if wrapped_stream:
             setattr(sys, stream_name, wrapped_stream)
+
+# Support FIPS enabled Python
+def md5_hasher():
+    try:
+        md5_hasher = hashlib.new('md5', usedforsecurity=False)
+    except:
+        md5_hasher = hashlib.new('md5')
+    return md5_hasher
