@@ -37,7 +37,7 @@ from . import logger
 
 UPDATABLE_GLOBALWF_FIELDS = {'title', 'summary', 'description', 'developerNotes', 'details'}
 GLOBALWF_SUPPORTED_KEYS = UPDATABLE_GLOBALWF_FIELDS.union({"name", "version", "regionalOptions",
-                                                           "categories", "billTo", "dxapi"})
+                                                           "categories", "billTo", "dxapi", "tags"})
 SUPPORTED_KEYS = GLOBALWF_SUPPORTED_KEYS.union({"project", "folder", "outputFolder", "stages",
                                                 "inputs", "outputs", "ignoreReuse"})
 
@@ -49,7 +49,7 @@ class WorkflowBuilderException(Exception):
     pass
 
 
-def _parse_executable_spec(src_dir, json_file_name, parser):
+def _fetch_spec_from_dxworkflowjson(src_dir, json_file_name, parser):
     """
     Returns the parsed contents of a json specification.
     Raises WorkflowBuilderException (exit code 3) if this cannot be done.
@@ -69,6 +69,49 @@ def _parse_executable_spec(src_dir, json_file_name, parser):
             raise WorkflowBuilderException("Could not parse {} file as JSON: {}".format(json_file_name, e.args))
 
 
+def _fetch_spec_from_workflow(args, parser):
+    try:
+        source_workflow = dxpy.DXWorkflow(args._from)
+        json_spec = source_workflow.describe(fields={"properties", "details"}, default_fields=True)
+        json_spec = _cleanup_empty_keys(json_spec)
+
+    except Exception as e:
+        raise WorkflowBuilderException("Could not get specs from given workflow {}: {}".format(args._from, e.args))
+
+    return json_spec
+
+def _cleanup_empty_keys(json_spec):
+    import re
+    clean_json = re.sub('\"\w*\": (null|\{\}|\"\"|\[\])(\,|)\s*','',json.dumps(json_spec)).replace(", }","}")
+    return json.loads(clean_json)
+
+def _check_dxcompiler_version(json_spec):
+    SUPPORTED_DXCOMPILER_VERSION = "2.8.0"
+    if  json_spec.get("details") and json_spec["details"].get("version"):
+        from distutils.version import StrictVersion
+        compiler_version_used = str(json_spec["details"].get("version"))
+        compiler_version_no_snapshot = compiler_version_used.split("-")[0]
+        if StrictVersion(compiler_version_no_snapshot) < StrictVersion(SUPPORTED_DXCOMPILER_VERSION):
+            raise WorkflowBuilderException("Source workflow {} is not compiled using dxCompiler (version>={}) that supports creating global workflows.".format(json_spec["name"], SUPPORTED_DXCOMPILER_VERSION))
+    else:
+        raise WorkflowBuilderException("Cannot find the dxCompiler version from the dxworkflow.json/source workflow spec. Please specify it by updating the details field of the dxworkflow.json/source workflow spec using the 'version' key.")
+
+def _notify_instance_type_selection(json_spec):
+    is_static = json_spec["details"].get("staticInstanceTypeSelection", False)
+    if is_static:
+        print("Note: {workflow} was compiled with -instanceTypeSelection=static "
+              "and will produce a global workflow that relies on instance types that may not be available to all users."
+              .format(workflow=json_spec["name"]))
+
+def _notify_dependencies(json_spec):
+    # APPS-622: A textual representation of all the dependencies is appended to 
+    # the “description” field of every workflow generated by the dxCompiler in markdown format.
+     dependency_list = json_spec.get("description")
+     if dependency_list:
+         print("Note: {workflow} was compiled with unbundled dependencies. "
+              "Please check the workflow description and make sure access to these dependencies is provided to all authorized users."
+              .format(workflow=json_spec["name"]))
+
 def _get_destination_project(json_spec, args, build_project_id=None):
     """
     Returns destination project in which the workflow should be created.
@@ -145,8 +188,11 @@ def _version_exists(json_spec, name=None, version=None):
     made a "describe" API call on the global workflow and so know the
     requested name and version already exists.
     """
-    requested_name = json_spec['name']
-    requested_version = json_spec['version']
+    try: 
+        requested_name = json_spec['name']
+        requested_version = json_spec['version']
+    except:
+        raise WorkflowBuilderException("Both 'name' and 'version' fields must be given in the dxworkflow.json/source workflow spec to build/update a global workflow.")
 
     if requested_name == name and requested_version == version:
         return True
@@ -236,13 +282,9 @@ def _validate_json_for_global_workflow(json_spec, args):
     Since building a global workflow is done after all the underlying workflows
     are built, which may be time-consuming, we validate as much as possible here.
     """
-    # TODO: verify the billTo can build the workflow
-    # TODO: if the global workflow build fails add an option to interactively change billto
-    # TODO: (or other simple fields) instead of failing altogether
-    # TODO: get a confirmation before building a workflow that may be costly
     if 'name' not in json_spec:
         raise WorkflowBuilderException(
-            "dxworkflow.json contains no 'name' field, but it is required to build a global workflow")
+            "dxworkflow.json/source workflow spec contains no 'name' field, but it is required to build a global workflow")
     if not dxpy.executable_builder.GLOBAL_EXEC_NAME_RE.match(json_spec['name']):
         raise WorkflowBuilderException(
             "The name of your workflow must match /^[a-zA-Z0-9._-]+$/")
@@ -251,7 +293,7 @@ def _validate_json_for_global_workflow(json_spec, args):
 
     if 'version' not in json_spec:
         raise WorkflowBuilderException(
-            "dxworkflow.json contains no 'version' field, but it is required to build a global workflow")
+            "dxworkflow.json/source workflow spec contains no 'version' field, but it is required to build a global workflow")
     if not dxpy.executable_builder.GLOBAL_EXEC_VERSION_RE.match(json_spec['version']):
         logger.warn('"version" {} should be semver compliant (e.g. of the form X.Y.Z)'.format(json_spec['version']))
 
@@ -267,13 +309,11 @@ def _validate_json_for_global_workflow(json_spec, args):
             raise WorkflowBuilderException(
                 'The field "regionalOptions" must be a non-empty dictionary whose values are dictionaries')
 
-    if args.bill_to:
-        json_spec["billTo"] = args.bill_to
 
 
 def _get_validated_json(json_spec, args):
     """
-    Validates dxworkflow.json and returns the json that can be sent with the
+    Validates workflow spec and returns the json that can be sent with the
     /workflow/new API or /globalworkflow/new request.
     """
     if not json_spec:
@@ -295,9 +335,9 @@ def _get_validated_json(json_spec, args):
         validated_spec['stages'] = _get_validated_stages(validated_spec['stages'])
 
     if 'name' in validated_spec:
-        if args.src_dir != validated_spec['name']:
+        if not args._from and args.src_dir != validated_spec['name']:
             logger.warn(
-                'workflow name "%s" does not match containing directory "%s"' % (validated_spec['name'], args.src_dir))
+                'workflow name "{}" does not match containing directory "{}"'.format(validated_spec['name'], args.src_dir))
 
     if 'ignoreReuse' in validated_spec:
         validate_ignore_reuse(validated_spec['stages'], validated_spec['ignoreReuse'])
@@ -325,13 +365,14 @@ def _get_validated_json_for_build_or_update(json_spec, args):
     """
     validated = copy.deepcopy(json_spec)
 
-    dxpy.executable_builder.inline_documentation_files(validated, args.src_dir)
+    if not args._from:
+        dxpy.executable_builder.inline_documentation_files(validated, args.src_dir)
 
     if 'title' not in json_spec:
-        logger.warn("dxworkflow.json is missing a title, please add one in the 'title' field")
+        logger.warn("dxworkflow.json/source workflow spec is missing a title, please add one in the 'title' field")
 
     if 'summary' not in json_spec:
-        logger.warn("dxworkflow.json is missing a summary, please add one in the 'summary' field")
+        logger.warn("dxworkflow.json/source workflow spec is missing a summary, please add one in the 'summary' field")
     else:
         if json_spec['summary'].endswith('.'):
             logger.warn("summary {} should be a short phrase not ending in a period".format(json_spec['summary'],))
@@ -341,37 +382,49 @@ def _get_validated_json_for_build_or_update(json_spec, args):
 
 def _assert_executable_regions_match(workflow_enabled_regions, workflow_spec):
     """
-    Check if the global workflow regions and the regions of stages (apps) match.
-    If the workflow contains any applets, the workflow can be currently enabled
+    Check if the dependent apps/applets/subworkflows in the workflow are enabled in requested regions
+    Returns the subset of requested regions where all the dependent executables are enabled
+    If the workflow contains any applets, then the workflow can be currently enabled
     in only one region - the region in which the applets are stored.
     """
-    executables = [i.get("executable") for i in workflow_spec.get("stages")]
+    if not workflow_enabled_regions: # empty set
+        return workflow_enabled_regions
+    
+    # get executable from all stages and sort them in the order app/applet/globalworkflow/workflow
+    executables = sorted([i.get("executable") for i in workflow_spec.get("stages")])
 
     for exect in executables:
-
-        if exect.startswith("applet-") and len(workflow_enabled_regions) > 1:
-            raise WorkflowBuilderException("Building a global workflow with applets in more than one region is not yet supported.")
+        if exect.startswith("applet-"):
+            applet_project = dxpy.DXApplet(exect).project
+            applet_region = dxpy.DXProject(applet_project).region
+            if applet_region in workflow_enabled_regions:                
+                workflow_enabled_regions = {applet_region} # only one region is allowed when using applet
+            else:
+                raise WorkflowBuilderException("The applet {} is not available in requested region(s) {}"
+                                               .format(exect, ','.join(workflow_enabled_regions)))
 
         elif exect.startswith("app-"):
             app_regional_options = dxpy.api.app_describe(exect, input_params={"fields": {"regionalOptions": True}})
             app_regions = set(app_regional_options['regionalOptions'].keys())
             if not workflow_enabled_regions.issubset(app_regions):
-                additional_workflow_regions = workflow_enabled_regions - app_regions
+                additional_workflow_regions = workflow_enabled_regions.difference(app_regions)
                 mesg = "The app {} is enabled in regions {} while the global workflow in {}.".format(
                     exect, ", ".join(app_regions), ", ".join(workflow_enabled_regions))
                 mesg += " The workflow will not be able to run in {}.".format(", ".join(additional_workflow_regions))
                 mesg += " If you are a developer of the app, you can enable the app in {} to run the workflow in that region(s).".format(
                     ", ".join(additional_workflow_regions))
                 logger.warn(mesg)
+                workflow_enabled_regions.intersection_update(app_regions)
 
         elif exect.startswith("workflow-"):
              # We recurse to check the regions of the executables of the inner workflow
             inner_workflow_spec = dxpy.api.workflow_describe(exect)
-            _assert_executable_regions_match(workflow_enabled_regions, inner_workflow_spec)
+            workflow_enabled_regions = _assert_executable_regions_match(workflow_enabled_regions, inner_workflow_spec)
 
         elif exect.startswith("globalworkflow-"):
             raise WorkflowBuilderException("Building a global workflow with nested global workflows is not yet supported")
 
+    return workflow_enabled_regions
 
 def _build_regular_workflow(json_spec, keep_open=False):
     """
@@ -383,54 +436,68 @@ def _build_regular_workflow(json_spec, keep_open=False):
     return workflow_id
 
 
-def _get_validated_enabled_regions(json_spec, from_command_line):
+def _get_validated_enabled_regions(json_spec, args):
     """
     Returns a set of regions (region names) in which the global workflow
-    should be enabled. Also validates and synchronizes the regions
-    passed via CLI argument and in the regionalOptions field.
+    should be enabled. 
+    
+    1. validates and synchronizes the regions passed via CLI argument and in the regionalOptions field.
+    2. checks if these regions are included in the permitted regions of the bill_to
+    3. checks if the dependencies in all the stages are enabled in these regions
     """
+    # Determine in which regions the global workflow are requested to be available
     enabled_regions = dxpy.executable_builder.get_enabled_regions('globalworkflow',
                                                                   json_spec,
-                                                                  from_command_line,
+                                                                  args.region,
                                                                   WorkflowBuilderException)
     if not enabled_regions:
         enabled_regions = []
-        if not dxpy.WORKSPACE_ID:
+        if not dxpy.PROJECT_CONTEXT_ID:
             msg = "A context project must be selected to enable a workflow in the project's region."
             msg += " You can use 'dx select' to select a project. Otherwise you can use --region option"
             msg += " to select a region in which the workflow should be enabled"
             raise(WorkflowBuilderException(msg))
-        region = dxpy.api.project_describe(dxpy.WORKSPACE_ID,
+        current_selected_region = dxpy.api.project_describe(dxpy.PROJECT_CONTEXT_ID,
                                            input_params={"fields": {"region": True}})["region"]
 
-        enabled_regions.append(region)
+        enabled_regions.append(current_selected_region)
 
+    # Get billable regions
+    enabled_regions = set(enabled_regions)
+    billable_regions = dxpy.executable_builder.get_permitted_regions(
+        json_spec["billTo"], WorkflowBuilderException)
+    if not enabled_regions.issubset(billable_regions):
+        raise WorkflowBuilderException("The global workflow cannot be enabled in regions {}, which are not among the permittedRegions of the billTo."
+                                       .format(",".join(enabled_regions.difference(billable_regions))))
+
+    # Verify dependencies in all the stages are also enabled in these regions
+    enabled_regions = _assert_executable_regions_match(enabled_regions, json_spec)
+    
     if not enabled_regions:
-        raise AssertionError("This workflow should be enabled in at least one region")
+        raise AssertionError("This workflow should be enabled in at least one region.")
 
-    return set(enabled_regions)
+    return enabled_regions
 
 
-def _create_temporary_projects(enabled_regions, args):
+def _create_temporary_projects(enabled_regions, bill_to):
     """
     Creates a temporary project needed to build an underlying workflow
     for a global workflow. Returns a dictionary with region names as keys
     and project IDs as values
 
     The regions in which projects will be created can be:
-    i. regions specified in dxworkflow.json "regionalOptions"
-    ii. regions specified as an argument to "dx build"
-    iii. current context project, if None of the above are set
-    If both args and dxworkflow.json specify regions, they must match.
+    i. regions specified in the dxworkflow.json/source workflow spec "regionalOptions"
+    ii. regions specified as the argument "--region" when calling "dx build"
+    iii. current context project, if none of the above are set
+    iv. the regions where dependent applets/apps/workflows are enabled
     """
     # Create one temp project in each region
     projects_by_region = {}  # Project IDs by region
     for region in enabled_regions:
         try:
             project_input = {"name": "Temporary build project for dx build global workflow",
-                             "region": region}
-            if args.bill_to:
-                project_input["billTo"] = args.bill_to
+                             "region": region,
+                             "billTo": bill_to}
             temp_project = dxpy.api.project_new(project_input)["id"]
             projects_by_region[region] = temp_project
             logger.debug("Created temporary project {} to build in".format(temp_project))
@@ -448,7 +515,7 @@ def _build_underlying_workflows(enabled_regions, json_spec, args):
     Returns a tuple of dictionaries: workflow IDs by region and project IDs by region.
     The caller is responsible for destroying the projects if this method returns properly.
     """
-    projects_by_region = _create_temporary_projects(enabled_regions, args)
+    projects_by_region = _create_temporary_projects(enabled_regions, json_spec["billTo"])
     workflows_by_region = {}
 
     try:
@@ -466,18 +533,11 @@ def _build_underlying_workflows(enabled_regions, json_spec, args):
     return workflows_by_region, projects_by_region
 
 
-def _build_global_workflow(json_spec, args):
+def _build_global_workflow(json_spec, enabled_regions, args):
     """
     Creates a workflow in a temporary project for each enabled region
     and builds a global workflow on the platform based on these workflows.
     """
-    # First determine in which regions the global workflow needs to be available
-    enabled_regions = _get_validated_enabled_regions(json_spec, args.region)
-
-    # Verify all the stages are also enabled in these regions
-    # TODO: Add support for dx building multi-region global workflows with applets
-    _assert_executable_regions_match(enabled_regions, json_spec)
-
     workflows_by_region, projects_by_region = {}, {}  # IDs by region
     try:
         # prepare "regionalOptions" field for the globalworkflow/new input
@@ -560,7 +620,7 @@ def _update_global_workflow(json_spec, args, global_workflow_id):
     validated_spec = _get_validated_json_for_build_or_update(update_spec, args)
     non_empty_fields = dict((k, v) for k, v in validated_spec.items() if v)
 
-    if not skip_update():
+    if args.update and not skip_update():
         global_workflow_id = dxpy.api.global_workflow_update('globalworkflow-' + json_spec['name'],
                                                              alias=json_spec['version'],
                                                              input_params=non_empty_fields)['id']
@@ -569,16 +629,36 @@ def _update_global_workflow(json_spec, args, global_workflow_id):
     return global_workflow_id
 
 
-def _build_or_update_workflow(json_spec, args):
+def _build_or_update_workflow(args, parser):
     """
     Creates or updates a workflow on the platform.
     Returns the workflow ID, or None if the workflow cannot be created.
     """
     try:
         if args.mode == 'workflow':
+            json_spec = _fetch_spec_from_dxworkflowjson(args.src_dir, "dxworkflow.json", parser)
             json_spec = _get_validated_json(json_spec, args)
             workflow_id = _build_regular_workflow(json_spec, args.keep_open)
         elif args.mode == 'globalworkflow':
+            if args._from:
+                json_spec = _fetch_spec_from_workflow(args, parser)
+            else:
+                json_spec = _fetch_spec_from_dxworkflowjson(args.src_dir, "dxworkflow.json", parser)
+            
+            # Override version number in json_spec if --version is specified
+            # version is required when building global workflow but is optional for workflow
+            # so `dx build` requires --version to be specified when using the --from
+            if args.version_override:
+                json_spec["version"] = args.version_override
+            json_spec = _get_validated_json(json_spec, args)
+            
+            # Check if the local or source workflow is compiled by dxCompiler that supported dependency annotation
+            if json_spec.get("tags") and "dxCompiler" in json_spec["tags"]:
+                _check_dxcompiler_version(json_spec)
+                if not args.brief:
+                    _notify_instance_type_selection(json_spec)
+                    _notify_dependencies(json_spec)
+            
             # Verify if the global workflow already exists and if the user has developer rights to it
             # If the global workflow name doesn't exist, the user is free to build it
             # If the name does exist two things can be done:
@@ -590,8 +670,9 @@ def _build_or_update_workflow(json_spec, args):
                                                      existing_workflow.version):
                 workflow_id = _update_global_workflow(json_spec, args, existing_workflow.id)
             else:
-                json_spec = _get_validated_json(json_spec, args)
-                workflow_id = _build_global_workflow(json_spec, args)
+                json_spec["billTo"] = dxpy.executable_builder.get_valid_bill_to(args.bill_to, WorkflowBuilderException)
+                enabled_regions = _get_validated_enabled_regions(json_spec, args)
+                workflow_id = _build_global_workflow(json_spec, enabled_regions, args)
         else:
             raise WorkflowBuilderException("Unrecognized workflow type: {}".format(args.mode))
     except dxpy.exceptions.DXAPIError as e:
@@ -619,9 +700,8 @@ def build(args, parser):
         raise Exception("Arguments not provided")
 
     try:
-        json_spec = _parse_executable_spec(args.src_dir, "dxworkflow.json", parser)
-        workflow_id = _build_or_update_workflow(json_spec, args)
+        workflow_id = _build_or_update_workflow(args, parser)
         _print_output(workflow_id, args)
     except WorkflowBuilderException as e:
-        print("Error: %s" % (e.args,), file=sys.stderr)
+        print("Error: {}".format(e.args,), file=sys.stderr)
         sys.exit(3)
